/*
 
 DESCRIPTION:    main include file TS350 inverter.
 
 COPYRIGHT:      (c) 2019, Brian Cornell, all rights reserved.
 
 HW REVISION:    1
 
 HW RELEASE:     190314
 
 VERSION:        2.0
 
 MODIFICATION HISTORY:
 
 19 Mar 19, BWC, created release.
 
 
 */

#define __MAIN_H                            //Do not edit


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Version constants & definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define VERMAJOR                1
#define VERMINOR                0
#define VERBUILD                14


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Necessary includes
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifndef UINT_MAX
#include <limits.h>
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Conditional compile features.  Uncomment each #define below to enable a 
 * feature.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#define _DEBUG                              //Uncomment for conditional testing
//#define _TESTPWM                            //Uncomment to use test pattern
//#define _SIMUL                              //Uncomment if using simulator
#define FEATURE_NTC                         //Thermal protection using NTC thermistor

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Regulate AC out (e.g. fixed HV DC PS).  If disabled the array objects in
 * array.h must still be defined but only object st0 must contain a valid
 * sine pattern.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define FEATURE_REG


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * System wide constants & definitions.  Convention:  S_<TYPE>_<INFO>
 * Types:
 *  + DC=Duty Cycle
 *  + EN=ENable bit
 *  + RT=RaTio of a voltage presented to PIC, internal or external
 *  + LM=LiMiting operational value
 *  + TM=TiMing
 *  + MP=Peripheral port bit MaPping
 *  + CH=character
 *  + IN=INitializing value
 *  + VL=VoLtage
 *  + QY=quantity (count) of something
 *  + CD=command data
 *  + BL=boolean value
 *  + MK=bit MasK
 * 
 * Info is user defined and should be descriptive.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Set these values according to DDS design in sine workbook
#define S_QY_PWMQSCYC           200     //PWM cycles in quarter sine (PWM Cycles / Sine Quarter)
#define S_QY_SINETABLE          8       //Number of sine arrays (# of regulation steps)
#define S_LM_AUXSAMPLES         120     //# aux volt samples to average (# samples to average)
#define S_VL_AUXMIN             313     //ADC value for min aux PS voltage (ADC value @ MIN sense)
#define S_VL_AUXINC             19      //ADC value per sine table increment (ADC value / step)

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * The DAC hot & overheat values must be set (calibrated) based on run-time 
 * evaluation.  Values should be set with a stable Vdd (the DAC & NTC are both
 * referenced to this).
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define S_LM_HOT             0x0D       //Hot threshold:  2V with Vref=5V
#define S_LM_OVH             0x0A       //Overheat threshold:  1.5V with Vref=5V

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *             DO NOT MODIFY THIS FILE BELOW THIS COMMENT
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define S_LM_SINETABLE          (S_QY_SINETABLE - 1)    //Sine table array limit
#define S_LM_PWMINDEX           (S_QY_PWMQSCYC - 1) //PWM index limit
#define S_VL_AUXMAX             ((S_LM_SINETABLE * S_VL_AUXINC) + S_VL_AUXMIN)  //ADC value for max aux PS voltage 
#define S_CD_RUN                1       //Sine wave generator start
#define S_CD_STOP               0       //Sine wave generator stop
#define S_BL_INCR               1       //Sine wave increasing
#define S_BL_DECR               0       //Sine wave decreasing
#define S_BL_POS                0       //Sine positive half cycle
#define S_BL_NEG                1       //Sine negative half cycle
#define S_TM_INT                100     //#mS to wait after initialization
#define S_LM_TMR2START          3       //# of periods to wait before enabling CWG
#define S_BL_HSON               PWM5OUT //TRUE if PWM output is high
#define S_EN_ADC                0x01    //Enable bit position in ADCON0 register
#define S_MP_HOT                C1OUT   //Comparator 1 output bit
#define S_MP_OVH                C1OUT   //Comparator 1 output bit
#define S_BL_HOT                FALSE   //Checking for HOT temp
#define S_BL_OVH                TRUE    //Checking for OverHeat temp
#define S_LM_HOT_HYS            100     //Hysteresis for hot temperature threshold
#define S_LM_OVH_HYS            100     //Hysteresis for overheattemperature threshold
#define S_TM_TIMEOUT            13      //# of loops to delay turn-on/off
#define S_LM_PCON_EX            0xFC    //PCON bits to examine at boot
#define S_LM_PCON_BT            0x1C    //PCON expected value at boot
#define S_LM_STAT_EX            0x18    //STATUS bits to examine at boot
#define S_LM_STAT_BT            0x18    //STATUS expected value at boot
#define S_IN_SAMPLE             1       //Starting value for sample counter
#define S_LM_SINE_HYS           15      //Hysteresis for updating sineSelect


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Analog channel mapping:  channel=port=pin.  The values assigned to these
 * constants should match those used by ADC and bit-aligned for direct loading
 * into ADC control register.  Channels available depend on specific chip.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define ANLCAN0                 0x00    //ANA0=RA0=13
#define ANLCAN1                 0x04    //ANA1=RA1=12
#define ANLCAN2                 0x08    //ANA2=RA2=11
#define ANLCAN3                 0x0C    //RESERVED
#define ANLCAN4                 0x10    //ANA4=RA4=3
#define ANLCAN5                 0x14    //ANA5=RA5=2
#define ANLCAN6                 0x30    //ANB4=n/a
#define ANLCAN7                 0x34    //ANB5=n/a
#define ANLCAN8                 0x38    //ANB6=n/a
#define ANLCAN9                 0x3C    //ANB7=n/a
#define ANLCAN10                0x40    //ANC0=RC0=10
#define ANLCAN11                0x44    //ANC1=RC1=9
#define ANLCAN12                0x48    //ANC2=RC2=8
#define ANLCAN13                0x4C    //ANC3=RC3=7
#define ANLCAN14                0x50    //ANC4=RC4=6
#define ANLCAN15                0x54    //ANC5=RC5=5
#define ANLCAN16                0x58    //ANC6=n/a
#define ANLCAN17                0x5C    //ANC7=n/a


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * PPS input pin definitions.  Map for MPU and use to assign peripheral input
 * pins.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PPSRA0      0x00                //RA0
#define PPSRA1      0x01                //RA1
#define PPSRA2      0x02                //RA2
#define PPSRA3      0x03                //RA3
#define PPSRA4      0x04                //RA4
#define PPSRA5      0x05                //RA5
#define PPSRB4      0x0C                //RB4
#define PPSRB5      0x0D                //RB5
#define PPSRB6      0x0E                //RB6
#define PPSRB7      0x0F                //RB7
#define PPSRC0      0x10                //RC0
#define PPSRC1      0x11                //RC1
#define PPSRC2      0x12                //RC2
#define PPSRC3      0x13                //RC3
#define PPSRC4      0x14                //RC4
#define PPSRC5      0x15                //RC5
#define PPSRC6      0x16                //RC6
#define PPSRC7      0x17                //RC7


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                           PPS & direct pin mapping
 * 
 * For PPS output pin assignments both aliases for pin registers and constants 
 * mapping what peripheral connects to the pin.
 *      Pin register alias convention:  PPS_P_XXX_YY where
 *          XXX=peripheral name
 *          YY=channel or other detail; can be omitted
 *      Peripheral mapping convention:  PPS_M_XXX_YY same as alias
 * 
 * Input pins only require mapping constant.
 * 
 * For pins accessed directly or assigned in a peripheral's register (like ADC)
 * use the following convention:  PIN_X_YYYYYY where
 *          X=Input or Output
 *          YYYYYY=user assigned mnemonic describing function
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PPS_P_CWG1_A            RA2PPS  //Output on pin 11
#define PPS_M_CWG1_A            0x08    //PPS mapping to peripheral
#define PPS_P_CWG1_B            RA4PPS  //Output on pin 3
#define PPS_M_CWG1_B            0x09    //PPS mapping to peripheral

#define PPS_M_CLC_0             0x02    //RA2, pin 11
#define PPS_M_CLC_1             0x04    //RA4, pin 3
#define PPS_M_CLC_2             0x10    //RC0, pin 10
#define PPS_M_CLC_3             0x15    //RC5, pin 5

#define PPS_P_CLC1              RC4PPS  //Output A_H
#define PPS_M_CLC1              0x04    //PPS mapping to peripheral
#define PPS_P_CLC2              RC3PPS  //Output A_L
#define PPS_M_CLC2              0x05    //PPS mapping to peripheral
#define PPS_P_CLC3              RC1PPS  //Output B_H
#define PPS_M_CLC3              0x06    //PPS mapping to peripheral
#define PPS_P_CLC4              RC2PPS  //Output B_L
#define PPS_M_CLC4              0x07    //PPS mapping to peripheral

#define PIN_O_ENA               RC0     //Enables A high-side and B low-side
#define PIN_O_ENB               RC5     //Enables B high-side and A low-side

#define PIN_I_RUN               RA3     //Commands sine wave start/stop
#define PIN_O_ACK               RA5     //Command acknowledgement & status


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Port/pin configuration:  I/O, analog/digital, slew rate, etc.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Port I/O:  1=input, 0=output
#define IO_PORTA                0b11001011
#define IO_PORTB                0b11111111
#define IO_PORTC                0b11000000
//Port Analog/Digital:  1=analog, 0=digital
#define AD_PORTA                0b11000011
#define AD_PORTB                0b11111111
#define AD_PORTC                0b11000000

//Port weak pull-up:  1=enabled, 0=disabled
#define WP_PORTA                0b00000000
#define WP_PORTB                0b00000000
#define WP_PORTC                0b00000000
//Open drain:  1=open drain, 0=push-pull
#define OD_PORTA                0b00000000
#define OD_PORTB                0b00000000
#define OD_PORTC                0b00000000
//Slew rate:  1=limited, 0=maximum
#define SR_PORTA                0b00000000
#define SR_PORTB                0b00000000
#define SR_PORTC                0b00000000
//Input level:  1=ST, 0=TLL
#define IL_PORTA                0b00000000
#define IL_PORTB                0b00000000
#define IL_PORTC                0b00000000
//Port values to initially set output latches to
#define VL_PORTA                0b00000000
#define VL_PORTB                0b00000000
#define VL_PORTC                0b00000000


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                              Macros
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SETDACHOT()         DACCON1 = S_LM_HOT
#define SETDACOVH()         DACCON1 = S_LM_OVH
#define SETACKOFF()         PIN_O_ACK = FALSE
#define SETACKON()          PIN_O_ACK = TRUE


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure system clock & watchdog timer.  Expects 1MHz HFINTOSC to be active at boot.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_OSC_SRC     0x0             //Nibble setting oscillator selection
#define CFG_OSC_DIV     0x0             //Nibble setting divider selection (1)
#define CFG_OSC_FRQ     0x5             //Nibble setting HFINTOSC freququency (24MHz)
#define CFG_OSC_OSCTUNE 0x00            //Adjustment to HFINTOSC to obtain 60Hz
#define _XTAL_FREQ      24000000        //Must match Fosc to use __delayxxx macros
#define CFG_WDT_CON     0x10            //Watchdog interval=256mS, off.


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * FVR & DAC configuration
 * FVR:  ADC +Vref
 * DAC:  ratiometric to Vdd for NTC temperature sensing
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_FVR_C               0x83        //Enabled, ADC FVR 4.096
#define FI_FVR_DLY              30          //Time delay in uS for FVR buffer to stabilize

//DAC:
#define CFG_DAC_CON0            0x80        //Enabled, Vref+=Vdd, Vref-=Vss


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * ADC configuration & acquisition constants:  dedicated to acquiring auxiliary
 * PS voltage.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Acquisition configuration & control
#define CFG_ADC_CON0            (ANLCAN0 | 0x01)    //ANA0, enabled
#define CFG_ADC_CON1            0xA3        //Right justified, FOSC/32, -Vref=Vss, +Vref=FVR
#define CFG_ADC_ACT             0x00        //No auto-conversion
#define CFG_ADC_AQT             20          //# uS required for sample & hold cap to acquire
#define CFG_ADC_V1V             250         //ADC value for 1V when Vref=4.096

//Result data object
#define ADC_SIZ_INT             2           //# bytes in ADC integer
#define ADC_POS_HIG             1           //ADC result high byte
#define ADC_POS_LOW             0           //ADC result low byte

//Internal channels (peripherals)
#define ADC_CH_VSS              0xF0        //Vss
#define ADC_CH_TMP              0xF4        //Temperature indicator
#define ADC_CH_DAC_1            0xF8        //DAC1
#define ADC_CH_FVR              0xFC        //FVR


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Comparator configuration constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_CON0             0x86        //Enable, hysteresis
#define CFG_CM_CON1             0x28        //+ = DAC, - = AN1 (C1IN0-)


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Timer2:  PWM5 time base interrupt for loading successive DC
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Timer2
#define CFG_TMR_2_CON           0x00    //No pre/post scale, off
#define CFG_TMR_2_FRQ           124     //48kHz


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * PWM configuration constants
 * PWM5:  digital sine wave modulation
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//PWM5
#define CFG_PWM_5_CON           0x00    //Disabled, active high output
#define CFG_PWM_5_TMRS          0x05    //TMR2 is period source
#define CFG_PWM_DCH_BITS        2
#define CFG_PWM_DCL_BITS        6
#define CFG_PWM_DCL_MASK        0x0003  


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Complementary waveform generator configuration constants
 * CWG1:  full bridge steering of PWM5 output
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//CWG1
#define CFG_CWG_1_CON0          0x04    //Disabled, half-bridge mode
#define CFG_CWG_1_CON1          0x00    //Normal polarity on all outputs
#define CFG_CWG_1_CLK           0x00    //Fosc is clock
#define CFG_CWG_1_DAT           0x07    //PWM5 is data input signal
#define CFG_CWG_1_AS0           0xA8    //Auto-shutdown ON, outputs use inactive state
#define CFG_CWG_1_AS1           0x00    //No HW auto-shutdown sources
#define CFG_CWG_1_DBR           0x02    //Rising dead-band delay
#define CFG_CWG_1_DBF           0x02    //Falling dead-band delay


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC1 as AND logic cell to control A high-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_1_CON       0x02    //Not enabled, no interrupts, AND mode
#define CFG_CLC_1_POL       0x00    //No inverted polarities
#define CFG_CLC_1_SEL0      0x00    //CLCIN0PPS
#define CFG_CLC_1_SEL1      0x00    //CLCIN0PPS
#define CFG_CLC_1_SEL2      0x02    //CLCIN2PPS
#define CFG_CLC_1_SEL3      0x02    //CLCIN2PPS
#define CFG_CLC_1_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_1_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_1_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_1_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC2 as AND-OR logic cell to control A low-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_2_CON       0x00    //Not enabled, no interrupts, AND-OR mode
#define CFG_CLC_2_POL       0x00    //No inverted polarities
#define CFG_CLC_2_SEL0      0x01    //CLCIN1PPS
#define CFG_CLC_2_SEL1      0x01    //CLCIN1PPS
#define CFG_CLC_2_SEL2      0x03    //CLCIN3PPS
#define CFG_CLC_2_SEL3      0x03    //CLCIN3PPS
#define CFG_CLC_2_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_2_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_2_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_2_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC3 as AND logic cell to control B high-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_3_CON       0x02    //Not enabled, no interrupts, AND mode
#define CFG_CLC_3_POL       0x00    //No inverted polarities
#define CFG_CLC_3_SEL0      0x00    //CLCIN0PPS
#define CFG_CLC_3_SEL1      0x00    //CLCIN0PPS
#define CFG_CLC_3_SEL2      0x03    //CLCIN3PPS
#define CFG_CLC_3_SEL3      0x03    //CLCIN3PPS
#define CFG_CLC_3_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_3_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_3_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_3_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC4 as AND-OR logic cell to control B low-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_4_CON       0x00    //Not enabled, no interrupts, AND-OR mode
#define CFG_CLC_4_POL       0x00    //No inverted polarities
#define CFG_CLC_4_SEL0      0x01    //CLCIN1PPS
#define CFG_CLC_4_SEL1      0x01    //CLCIN1PPS
#define CFG_CLC_4_SEL2      0x02    //CLCIN2PPS
#define CFG_CLC_4_SEL3      0x02    //CLCIN2PPS
#define CFG_CLC_4_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_4_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_4_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_4_LS_3      0x80    //d4T is gated into g4


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          Object definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Object to store a measured voltage with 3-digit mV resolution
typedef struct {
    unsigned int vmv;                   //Volts in 10K/1K places, mV in 100/10/1
    unsigned int volt;                  //Volts as integer
    unsigned int mv;                    //mV as integer 
} volts;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Status codes (currently not used)
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define STS_BOOT        0x00            //Startup
#define STS_RUN         0x01            //Running
#define STS_BUS_UVLO    0x02            //Stopped - bus under-volt lockout
#define STS_CHL_OCLO    0x03            //Stopped - over-current lockout
#define STS_OUT_UVLO    0x04            //Stopped - output under-volt lockout
#define STS_OUT_OVLO    0x05            //Stopped - output over-volt lockout
#define STS_BUS_OVLO    0x06            //Disabled - bus over-volt lockout

#define STS_CODE_ERR    0x02            //Codes < this value are not errors


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Function declarations.  See source file for explanation of each function.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void interrupt InterruptExec(void);
void Initialize(void);
void main(void);
void sineON(void);
void sineOFF(void);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Preprocessor checks.  Maintain at end of main include file.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



//End of file
